%%% -------------------------------------------------------------------
%%% Description :
%%%
%%% Created : 27.04.2012
%%% -------------------------------------------------------------------
-module(m).
%%% Author  : rekby

-behaviour(gen_server).
%% --------------------------------------------------------------------
%% Include files
%% --------------------------------------------------------------------
-define(TABLE_COUNT, 100).
-define(TABLE_PREFIX, "mess_").

%% --------------------------------------------------------------------
%% External exports
-export([get_count/0, put/1, process/2, start_link/0]).

%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-record(state, {table_name, list=[], count=0}).

%% ====================================================================
%% External functions
%% ====================================================================
get_count(0)->
	0;
get_count(TableNumber)->
	StringNumber = integer_to_list(TableNumber),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_existing_atom(TableName),
	[{_, Count}|_] = ets:lookup(state, TableAtom),
	Count + get_count(TableNumber - 1).


get_count()->
	get_count(?TABLE_COUNT).

put(Data)->
	TableNumber = random:uniform(?TABLE_COUNT),
	StringNumber = integer_to_list(TableNumber),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_existing_atom(TableName),
	gen_server:cast(TableAtom, {put, Data}).

process(Key, F) ->
	DataList = ets:lookup(et, Key),
	ets:delete(et, Key),
	F(DataList).

start_link()->
	CreateOptions = [named_table, set, public, {write_concurrency, true}, {read_concurrency, true}],
	%create_table(CreateOptions, ?TABLE_COUNT),
	ets:new(state, CreateOptions),
	create_table(?TABLE_COUNT),
	{ok, self()}.

%% ====================================================================
%% Server functions
%% ====================================================================

%% --------------------------------------------------------------------
%% Function: init/1
%% Description: Initiates the server
%% Returns: {ok, State}          |
%%          {ok, State, Timeout} |
%%          ignore               |
%%          {stop, Reason}
%% --------------------------------------------------------------------
init([TableName]) ->
    ets:insert(state, {TableName, 0}),
	{ok, #state{table_name=TableName}}.

%% --------------------------------------------------------------------
%% Function: handle_call/3s
%% Description: Handling call messages
%% Returns: {reply, Reply, State}          |
%%          {reply, Reply, State, Timeout} |
%%          {noreply, State}               |
%%          {noreply, State, Timeout}      |
%%          {stop, Reason, Reply, State}   | (terminate/2 is called)
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_call(_Request, _From, State) ->
    Reply = ok,
    {reply, Reply, State}.

%% --------------------------------------------------------------------
%% Function: handle_cast/2
%% Description: Handling cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_cast({put, Data}, State) ->
	NewState = State#state{list=[Data|State#state.list], count=State#state.count+1},
	%NewState = State#state{count=State#state.count+1},
	case (NewState#state.count rem 1000) of
		0 ->
			%io:format("~p-~p~n", [NewState#state.table_name, NewState#state.count]),
			ets:insert(state, {NewState#state.table_name, NewState#state.count});
		_ -> 
			ok
	end,
    {noreply, NewState}.

%% --------------------------------------------------------------------
%% Function: handle_info/2
%% Description: Handling all non call/cast messages
%% Returns: {noreply, State}          |
%%          {noreply, State, Timeout} |
%%          {stop, Reason, State}            (terminate/2 is called)
%% --------------------------------------------------------------------
handle_info(_Info, State) ->
    {noreply, State}.

%% --------------------------------------------------------------------
%% Function: terminate/2
%% Description: Shutdown the server
%% Returns: any (ignored bys gen_server)
%% --------------------------------------------------------------------
terminate(_Reason, _State) ->
    ok.

%% --------------------------------------------------------------------
%% Func: code_change/3
%% Purpose: Convert process state when code is changed
%% Returns: {ok, NewState}
%% --------------------------------------------------------------------
code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% --------------------------------------------------------------------
%%% Internal functions
%% --------------------------------------------------------------------
create_table(0)->
	ok;
create_table(Number)->
	StringNumber = integer_to_list(Number),
	TableName = ?TABLE_PREFIX ++ StringNumber,
	TableAtom = list_to_atom(TableName),
	gen_server:start_link({local, TableAtom}, ?MODULE, [TableAtom], []),
	create_table(Number-1).
